<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="GVPart" version="6" translationDomain="gwenview">
<MenuBar>
	<Menu name="file"><text>&amp;File</text>
		<Action name="file_save_as"/>
		<Action name="file_show_properties"/>
	</Menu>
	<Menu name="view"><text>&amp;View</text>
		<Action name="view_actual_size"/>
		<Action name="view_zoom_to_fit"/>
		<Action name="view_zoom_in"/>
		<Action name="view_zoom_out"/>
	</Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
	<Action name="view_actual_size"/>
	<Action name="view_zoom_to_fit"/>
	<Action name="view_zoom_in"/>
	<Action name="view_zoom_out"/>
</ToolBar>
</gui>
