// vim: set tabstop=4 shiftwidth=4 expandtab:
/*
Gwenview: an image viewer
Copyright 2021 Antonio Prcela <antonio.prcela@gmail.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA.

*/
#ifndef FULLSCREENBACKGROUND_H
#define FULLSCREENBACKGROUND_H

// Qt

// KF

// Local

namespace Gwenview
{
namespace FullScreenBackground
{
/**
 * This enum represents the different background types in fullscreen mode
 */
enum Enum {
    Black,
    Image,
};

} // namespace FullScreenBackground

} // namespace Gwenview

#endif /* FULLSCREENBACKGROUND_H */
